package com.amaps.location;

import android.content.Context;
import android.content.Intent;
import android.location.LocationManager;
import android.provider.Settings;
import androidx.annotation.NonNull;
import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationListener;
import java.util.HashMap;
import java.util.Map;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/** 高德地图定位插件 */
public class AmapsLocationPlugin implements FlutterPlugin, MethodCallHandler {
  public final static String TAG = "AmapsLocationPlugin";

  private MethodChannel channel;

  //声明AMapLocationClient类对象
  public AMapLocationClient mLocationClient = null;

  private Context mContext;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    mContext = flutterPluginBinding.getApplicationContext();
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "amaps/flutter.location.method");
    channel.setMethodCallHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
    if (call.method.equals("isOpenGPS")) {
      LocationManager locationManager = (LocationManager) mContext.getSystemService(Context.LOCATION_SERVICE);
      boolean gps = locationManager.isProviderEnabled(android.location.LocationManager.GPS_PROVIDER);
      boolean network = locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
      if (gps || network) {
        result.success(true);
      }else{
        result.success(false);
      }
    }else if (call.method.equals("enableGPS")) {
      Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
      intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK );
      mContext.startActivity(intent);
      result.success(true);
    } else if (call.method.equals("getLocation")) {
      if(mLocationClient == null){
        //初始化定位
        mLocationClient = new AMapLocationClient(mContext);
      }
      //设置定位回调监听
      mLocationClient.setLocationListener(new AMapLocationListener() {
        @Override
        public void onLocationChanged(AMapLocation location) {
          mLocationClient.stopLocation();
          Map<String,Object> locationMap = new HashMap<>();
          locationMap.put("errorCode",location.getErrorCode());
          locationMap.put("errorInfo",location.getErrorInfo());
          locationMap.put("locationDetail",location.getLocationDetail());
          locationMap.put("locationType",location.getLocationType());
          locationMap.put("gpsAccuracyStatus",location.getGpsAccuracyStatus());
          locationMap.put("aoiName",location.getAoiName());
          locationMap.put("poiName",location.getPoiName());
          locationMap.put("adCode",location.getAdCode());
          locationMap.put("cityCode",location.getCityCode());
          locationMap.put("streetNum",location.getStreetNum());
          locationMap.put("street",location.getStreet());
          locationMap.put("district",location.getDistrict());
          locationMap.put("city",location.getCity());
          locationMap.put("province",location.getProvince());
          locationMap.put("country",location.getCountry());
          locationMap.put("address",location.getAddress());
          locationMap.put("floor",location.getFloor());
          locationMap.put("buildingId",location.getBuildingId());
          locationMap.put("bearing",location.getBearing());
          locationMap.put("speed",location.getSpeed());
          locationMap.put("altitude",location.getAltitude());
          locationMap.put("accuracy",location.getAccuracy());
          locationMap.put("longitude",location.getLongitude());
          locationMap.put("latitude",location.getLatitude());
          result.success(locationMap);
        }
      });
      AMapLocationClientOption option = new AMapLocationClientOption();
      /// 设置定位场景，目前支持三种场景（签到、出行、运动，默认无场景）
      option.setLocationPurpose(AMapLocationClientOption.AMapLocationPurpose.SignIn);
      ///选择定位模式
      /// 设置定位模式为AMapLocationMode.Hight_Accuracy，高精度模式。
      /// 设置定位模式为AMapLocationMode.Battery_Saving，低功耗模式。
      /// 设置定位模式为AMapLocationMode.Device_Sensors，仅设备模式。

      option.setLocationMode(AMapLocationClientOption.AMapLocationMode.Hight_Accuracy);
      /// 设置是否开启定位缓存机制
      option.setLocationCacheEnable(false);
      mLocationClient.setLocationOption(option);
      mLocationClient.startLocation();
      //result.error("100","没有定位权限","定位权限没有开启，请开启定位权限");
    } else {
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }
}
