#import "AmapsLocationPlugin.h"
#import <AMapFoundationKit/AMapFoundationKit.h>
#import <AMapLocationKit/AMapLocationKit.h>

@interface AmapsLocationPlugin()<AMapLocationManagerDelegate>

@property (nonatomic, copy) AMapLocatingCompletionBlock completionBlock;

@property(nonatomic, retain) AMapLocationManager *locationManager;

@property(nonatomic) FlutterResult locationResult;

@end



@implementation AmapsLocationPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"amaps/flutter.location.method" binaryMessenger:[registrar messenger]];
  AmapsLocationPlugin* instance = [[AmapsLocationPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"isOpenGPS" isEqualToString:call.method]) {
      if ([CLLocationManager locationServicesEnabled] == YES){
          result(@YES);
      }else{
          result(@NO);
      }
  } else if ([@"enableGPS" isEqualToString:call.method]) {
      NSURL *url = [NSURL URLWithString:UIApplicationOpenSettingsURLString];
      BOOL isOpen = NO;
      if ([[UIApplication sharedApplication] canOpenURL:url]) {
          isOpen = [[UIApplication sharedApplication] openURL:url];
      }
      result(@(isOpen));
  } else if ([@"getLocation" isEqualToString:call.method]) {
      self.locationResult = result;
      NSString *apiKey = call.arguments[@"apiKey"];
      if(!self.locationManager){
          [[AMapServices sharedServices] setEnableHTTPS:YES];
          [[AMapServices sharedServices] setApiKey:apiKey];
          self.locationManager = [[AMapLocationManager alloc] init];
          // 带逆地理信息的一次定位（返回坐标和地址信息）
          [self.locationManager setDesiredAccuracy:kCLLocationAccuracyHundredMeters];
          //   定位超时时间，最低2s，此处设置为10s
          self.locationManager.locationTimeout =10;
          //   逆地理请求超时时间，最低2s，此处设置为10s
          self.locationManager.reGeocodeTimeout = 10;
      }
      self.locationManager.delegate = self;
      CLAuthorizationStatus authorizationStatus = [CLLocationManager authorizationStatus];
      NSLog(@"======>  Status:%d",authorizationStatus);
      if (authorizationStatus == kCLAuthorizationStatusNotDetermined) {
          self.locationResult(@{@"errorCode":@20,@"errorInfo":@"需要申请定位权限。"});
      } else if(authorizationStatus == kCLAuthorizationStatusDenied){
          NSLog(@"======>  缺少定位权限。。。。。拒绝了定位权限");
          self.locationResult(@{@"errorCode":@12,@"errorInfo":@"请在设备的设置中开启app的定位权限。"});
      }else if(authorizationStatus == kCLAuthorizationStatusAuthorized){
          [self getLocation:YES result:result];
      }else{
          [self getLocation:YES result:result];
      }
  } else {
    result(FlutterMethodNotImplemented);
  }
}

-(void)getLocation:(BOOL)withReGeocode result:(FlutterResult)result{
    self.completionBlock = ^(CLLocation *location, AMapLocationReGeocode *regeocode, NSError *error){
        if (regeocode){
            NSLog(@"reGeocode:%@", regeocode);
        }else{
            NSLog(@"=======> 无");
        }
        if (error != nil && error.code == AMapLocationErrorLocateFailed){
            //定位错误：此时location和regeocode没有返回值，不进行annotation的添加
            NSLog(@"定位错误:{%ld - %@};", (long)error.code, error.localizedDescription);
            result(@{ @"code":@(error.code),@"description":error.localizedDescription, @"success":@NO });
            return;
        }else if (error != nil && (error.code == AMapLocationErrorReGeocodeFailed
                             || error.code == AMapLocationErrorTimeOut
                             || error.code == AMapLocationErrorCannotFindHost
                             || error.code == AMapLocationErrorBadURL
                             || error.code == AMapLocationErrorNotConnectedToInternet
                             || error.code == AMapLocationErrorCannotConnectToHost)){
            //逆地理错误：在带逆地理的单次定位中，逆地理过程可能发生错误，此时location有返回值，regeocode无返回值，进行annotation的添加
            NSLog(@"逆地理错误:{%ld - %@};", (long)error.code, error.localizedDescription);
        }else if (error != nil && error.code == AMapLocationErrorRiskOfFakeLocation){
            //存在虚拟定位的风险：此时location和regeocode没有返回值，不进行annotation的添加
            NSLog(@"存在虚拟定位的风险:{%ld - %@};", (long)error.code, error.localizedDescription);
            result(@{ @"code":@(error.code),@"description":error.localizedDescription, @"success":@NO  });
            return;
        }else{
            //没有错误：location有返回值，regeocode是否有返回值取决于是否进行逆地理操作，进行annotation的添加
        }
        
        
        NSMutableDictionary* md = [[NSMutableDictionary alloc]initWithDictionary: [AmapsLocationPlugin location2map:location]];
        if(regeocode){
            [md addEntriesFromDictionary:[ AmapsLocationPlugin regeocode2map:regeocode ]];
        }
        md[@"success"]=@YES;
        result(md);
    };
    [self.locationManager requestLocationWithReGeocode:withReGeocode completionBlock:self.completionBlock];
}

- (void)amapLocationManager:(AMapLocationManager *)manager didUpdateLocation:(CLLocation *)location reGeocode:(AMapLocationReGeocode *)reGeocode{
    if(self.locationManager){
        [self.locationManager stopUpdatingLocation];
    }
}


+(NSDictionary*)regeocode2map:(AMapLocationReGeocode *)regeocode{
    return @{@"address":regeocode.formattedAddress,
             @"country":regeocode.country,
             @"province":regeocode.province,
             @"city":regeocode.city,
             @"district":regeocode.district,
             @"citycode":regeocode.citycode,
             @"adcode":regeocode.adcode,
             @"street":regeocode.street,
             @"number":regeocode.number,
             @"poiName":[self checkNull : regeocode.POIName],
             @"aoiName":[self checkNull :regeocode.AOIName],
             };
}

+(NSDictionary*)location2map:(CLLocation *)location{
    
    return @{@"latitude": @(location.coordinate.latitude),
             @"longitude": @(location.coordinate.longitude),
             @"accuracy": @((location.horizontalAccuracy + location.verticalAccuracy)/2),
             @"altitude": @(location.altitude),
             @"speed": @(location.speed),
             @"timestamp": @(location.timestamp.timeIntervalSince1970),};
    
}

+(id)checkNull:(NSObject*)value{
    return value == nil ? [NSNull null] : value;
}
@end
