import 'package:flutter/services.dart';

import 'location_entity.dart';

abstract class AmapsController {
  ///
  /// 获取GPS定位
  /// [apiKey]        高德地图官方apiKey
  ///
  Future<LocationEntity> getLocation({String? apiKey});

  ///
  /// 是否打开GPS
  ///
  Future<bool?> isOpenGPS();

  ///
  /// 启用手机GPS
  ///
  Future<bool?> enableGPS();
}

class PluginController implements AmapsController {
  final MethodChannel _amapsChannel;

  PluginController({
    required MethodChannel amapsChannel,
  }) : _amapsChannel = amapsChannel;

  @override
  Future<bool?> enableGPS() async {
    return await _amapsChannel.invokeMethod('enableGPS');
  }

  @override
  Future<LocationEntity> getLocation({String? apiKey}) async {
    LocationEntity entity = await _amapsChannel
        .invokeMethod('getLocation', {'apiKey': apiKey})
        .then((value) {
          print(value);
          return value;
        })
        .asStream()
        .map(LocationEntity.fromJson)
        .first;
    return entity;
  }

  @override
  Future<bool?> isOpenGPS() async {
    return await _amapsChannel.invokeMethod('isOpenGPS');
  }
}

class PluginControllerFactory {
  const PluginControllerFactory();

  PluginController create() {
    const MethodChannel _channelMethod =
        const MethodChannel('amaps/flutter.location.method');

    PluginController _controller = PluginController(
      amapsChannel: _channelMethod,
    );
    return _controller;
  }
}
